# -*- coding: utf-8 -*-

from django.core.management.base import CommandError
from fields import (NdaField, IntegerNda, FloatFieldNda, BooleanNda, CharNda,
                    DateNda, DateTimeNda, TimeNda, EmailNda, IPAdressNda,
                    NullBooleanNda, URLNda, SlugNda)
from django.db.models import fields as mfields

class NdaModel(object):

    def __init__(self, instance=None):
        if not isinstance(instance, self.Meta.model):
            raise Exception("`instance` parameter is not an instance of %s class" % self.__class__.__name__)
        self.instance = instance

    @classmethod
    def fields_for_nda(cls):
        """
        Return a ``List`` contaiining field for the model
        that declared in Meta

        if in Meta ``fields`` or/and ``exclude``:

        ``fields`` is an optional list of field names. If provided, only the named
        fields will be included in the returned fields.

        ``exclude`` is an optional list of field names. If provided, the named
        fields will be excluded from the returned fields, even if they are listed
        in the ``fields`` argument.
        """
        try:
            model = cls.Meta.model
        except AttributeError:
            raise CommandError("Specify model in `Meta` class")

        fields = getattr(cls.Meta, 'fields', None)
        exclude = getattr(cls.Meta, 'exclude', None)
        opts = model._meta
        field_list = []

        for f in opts.fields:
            if fields is not None and not f.name in fields:
                continue
            if exclude and f.name in exclude:
                continue
            if f.auto_created:
                continue
            if f.rel:
                continue
            field_list.append(f)

        return field_list

    @classmethod
    def excluded_fields(cls):
        ffnda = set([i.name for i in cls.fields_for_nda()])
        try:
            allfields = set([i.name for i in cls.Meta.model._meta.fields])
        except:
            raise CommandError("Specify model in `Meta` class")

        return allfields - ffnda

    @classmethod
    def map_fields(cls):
        """
        mapping django.models to NdaModels
        Return dict {'field': NdaModelField instance, ...}
        """
        fields = cls.fields_for_nda()

        res = {}
        for f in fields:
            current = getattr(cls, f.name, None)
            if current:
                res[f.name] = current
                continue

            if isinstance(f, mfields.BigIntegerField):
                res[f.name] = IntegerNda(min_value=-9223372036854775808,
                                         max_value=9223372036854775807)

            elif isinstance(f, mfields.BooleanField):
                res[f.name] = BooleanNda()

            elif isinstance(f, (mfields.CharField,)):
                res[f.name] = CharNda(max_length=f.max_length)

            elif isinstance(f, mfields.SlugField):
                res[f.name] = SlugNda(max_length=f.max_length)

            elif isinstance(f, mfields.DateField):
                res[f.name] = DateNda()

            elif isinstance(f, mfields.DateTimeField):
                res[f.name] = DateTimeNda()

            elif isinstance(f, mfields.EmailField):
                res[f.name] = EmailNda()

            elif isinstance(f, (mfields.DecimalField, mfields.FloatField)):
                res[f.name] = FloatFieldNda()

            elif isinstance(f, (mfields.PositiveIntegerField,
                                mfields.IntegerField,
                                mfields.SmallIntegerField)):
                res[f.name] = IntegerNda()

            elif isinstance(f, mfields.IPAddressField):
                res[f.name] = IPAdressNda()

            elif isinstance(f, mfields.NullBooleanField):
                res[f.name] = NullBooleanNda()

            elif isinstance(f, mfields.TextField):
                res[f.name] = CharNda(max_length=300)

            elif isinstance(f, mfields.TimeField):
                res[f.name] = TimeNda()

            elif isinstance(f, mfields.URLField):
                res[f.name] = URLNda()

            else:
                res[f.name] = NdaField()

        return res

    def obfuscation(self):
        rules = self.map_fields()
        for field, rule  in rules.items():
            attr = getattr(self.instance, field)
            setattr(self.instance, field, rule.obfuscate(attr))
        self.instance.save()



# -*- coding: utf-8 -*-

from __future__ import with_statement
import os
from random import randint, randrange, choice, random
from datetime import datetime, date, time

from hashlib import md5


BASE_PATH = os.path.split(os.path.abspath(__file__))[0]
BASE_PATH = os.path.join(BASE_PATH, '../')


class NdaField(object):
    DOMAINS = ['example.com', 'test.ok', 'ololo.gg', 'somebody.name', 'localhost.by']

    def __init__(self, source_file=None):
        if source_file:
            with open(source_file) as f:
                self.source = f.read().splitlines()

    def obfuscate(self, value):
        return value


class IntegerNda(NdaField):
    def __init__(self, min_value=None, max_value=None):
        self.min = min_value
        self.max = max_value

    def obfuscate(self, value):
        if not (self.min and self.max):
            len_ = len(str(value)) - 1
            res = (randint(10 ** len_, 10 ** (len_ + 1) - 1) *
                   (choice([-1, 1]) if value < 0 else 1))
        else:
            res = randint(self.min, self.max)

        return res


class FloatFieldNda(IntegerNda):
    def __init__(self, min_value=None, max_value=None):
        self.min = int(min_value)
        self.max = int(max_value) - 1

    def obfuscate(self, value):
        intg = super(FloatFieldNda, self).obfuscate(value)
        return intg + random()


class BooleanNda(NdaField):
    def obfuscate(self, value):
        return bool(randint(0, 1))


class CharNda(NdaField):
    def __init__(self, source_file=BASE_PATH+'texts/lorem.txt',
                 min_length=None, max_length=None, words=0):
        super(CharNda, self).__init__(source_file)
        self.min = min_length
        self.max = max_length
        self.words = words

    def obfuscate(self, value):
        text = '\n'.join(self.source)

        if self.words:
            res = []
            text = text.split()
            for _ in xrange(self.words):
                res.append(text[randint(0, len(text) - 1)])
            return ' '.join(res)

        if self.min and self.max:
            res = text[:randint(self.min, self.max)]
        elif self.max:
            res = text[:randint(1, self.max / 2)]
        elif self.min:
            res = text[:randint(self.min, len(text))]
        else:
            self.words = 2
            res = CharNda.obfuscate(self, value)
        return res


class SlugNda(CharNda):
    def obfuscate(self, value):
        self.max = self.max or 50
        self.words = self.words or 2
        text = '\n'.join(self.source).split()
        sep = choice(['', '-', '_'])
        res = []
        for _ in xrange(self.words):
            res.append(text[randint(0, len(text) - 1)])
        return sep.join(res)[:self.max]


class FirstNameNda(NdaField):
    def __init__(self, source_file=BASE_PATH+'texts/names.txt', sep=' '):
        super(FirstNameNda, self).__init__(source_file)
        self.sep = sep
        # part: 0 - firstname, 1 - lastname, 2 - middlename
        self.part = 0

    def obfuscate(self, value):
        """ source file shuld be in
        <firstname><separator><lastname><separator>[middlename]
        """
        text_lines = self.source
        return text_lines[
            randint(0, len(text_lines)-1)].split(self.sep)[self.part].strip()


class LastNameNda(FirstNameNda):
    def __init__(self, source_file=BASE_PATH+'texts/names.txt', sep=' '):
        super(LastNameNda, self).__init__(source_file, sep)
        # part: 0 - firstname, 1 - lastname, 2 - middlename
        self.part = 1


class MiddleNameNda(FirstNameNda):
    def __init__(self, source_file=BASE_PATH+'texts/names.txt', sep=' '):
        super(MiddleNameNda, self).__init__(source_file, sep)
        # part: 0 - firstname, 1 - lastname, 2 - middlename
        self.part = 2


class LoginNda(FirstNameNda):
    def __init__(self, source_file=BASE_PATH+'texts/login.txt', sep=' ',
                 unique=False):
        super(LoginNda, self).__init__(source_file, sep)
        self.unique = unique

    def obfuscate(self, value):
        text_lines = self.source
        if self.unique:
            postf = str(int(md5(str(value)).hexdigest(), 16) % 1000)
        else:
            postf = ''
        return text_lines[randint(0, len(text_lines)-1)].strip() + postf


class DateNda(NdaField):
    def obfuscate(self, value):
        start_date = date.today().replace(year=1901).toordinal()
        end_date = date.today().toordinal()
        return date.fromordinal(randint(start_date, end_date))


class DateTimeNda(DateNda):
    def obfuscate(self, value):
        rnd = super(DateTimeNda, self).obfuscate(value)
        return datetime(year=rnd.year, month=rnd.month, day=rnd.day,
                        hour=randrange(12), minute=randrange(60),
                        second=randrange(60))


class TimeNda(NdaField):
    def obfuscate(self, value):
        return time(hour=randrange(12), minute=randrange(60),
                    second=randrange(60))


class EmailNda(LoginNda):
    def obfuscate(self, value):
        last = choice(self.DOMAINS)
        first = super(LoginNda, self).obfuscate(value)
        return first + u'@' + last


class IPAdressNda(NdaField):
    def obfuscate(self, value):
        return u'.'.join([unicode(randint(1, 255)) for i in range(4)])


class NullBooleanNda(NdaField):
    def obfuscate(self, value):
        return choice([True, False, None])


class URLNda(NdaField):
    def obfuscate(self, value):
        value = value or ''
        h = md5(str(datetime.now()) + value).hexdigest()[:16]
        return u'http://%s/%s' %(choice(self.DOMAINS), h)


class HashNda(NdaField):
    def obfuscate(self, value):
        return md5(str(value)).hexdigest()



# -*- coding: utf-8 -*-

from django.core.management.base import NoArgsCommand, CommandError
from django.db import models

from optparse import make_option
from ndator.nda import NdaModel
import inspect


def get_nda_models():
    """ Return list of NdaModel classes
    from ndamodels.py file
    """
    class_list = []
    try:
        import ndamodels
    except ImportError:
        raise CommandError("Create ndamodels.py in project dir or use --allauto option.")

    for elem in dir(ndamodels):
        obj = getattr(ndamodels, elem)

        if (inspect.isclass(obj) and
            issubclass(obj, NdaModel) and
            obj is not NdaModel):
            class_list.append(obj)

    return class_list

def autoconvert_to_nda(model):
    """ Auto convert django model to NdaModel """
    meta = type('Meta', (), {'model': model})
    return type(model.__name__ + 'Nda', (NdaModel,), {'Meta': meta})


class Command(NoArgsCommand):
    help = ""
    requires_model_validation = False
    db_module = 'django.db'

    option_list = NoArgsCommand.option_list + (
        make_option('--allauto', action='store_true', dest='allauto',
                    default=False, help='Try to make obfuscation with default'
                    'settings'),
        )

    def handle_noargs(self, **options):
        allauto = options.get('allauto')
        answ = ''
        msg = ("At this step all information in models will be obfuscate"
               "\nAre you realy shure? (yes/no): ")
        answ = raw_input(msg).upper()
        while answ != 'YES':
            if answ == 'NO':
                return
            elif answ != 'YES':
                answ = raw_input('Please enter either "yes" or "no": ').upper()
        else:
            print

        if not allauto:
            models_for_nda = get_nda_models()
        else:
            models_for_nda = [autoconvert_to_nda(m) for m in models.get_models()]

        excluded = {}
        for m in models_for_nda:
            name = m.Meta.model.__name__

            # collect excluded fields
            exc = m.excluded_fields()
            if exc:
                excluded[name] = exc

            div = int(m.Meta.model.objects.count() / 20) + 1
            objects = m.Meta.model.objects.all()

            # main loop
            print name,
            for i, obj in enumerate(objects):
                m(obj).obfuscation()

                # progressbar :)
                if not i % div:
                    print '.',
            else:
                print

        print
        # models and fields that didn't obfuscate
        all_models = set([m.__name__ for m in models.get_models()])
        use_models = set([m.Meta.model.__name__ for m in models_for_nda])
        excluded_models = all_models - use_models


        # display excluded
        print 'This fields was not obfuscated:'
        for name, items in excluded.items():
            print name
            for item in sorted(items):
                print '    ' + item
            print

        print 'Models that was not obfuscated:'
        for model in excluded_models:
            print model



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

version = __import__('ndator').__version__

setup(
    name='django-ndator',
    version=version,
    author='Nikita Zubkov',
    author_email='zubchick@gmail.com',
    url='http://github.com/zubchick/django-ndator',
    description='Django models obfuscator',
    long_description=open('README.rst').read(),
    license='New BSD License',
    packages=find_packages(),
    package_data = {'ndator': ['texts/*.txt']},
    zip_safe=False,
    include_package_data=True,

    classifiers=[
        'Framework :: Django',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        ],
)
